@@ -43,6 +43,7 @@
 #include <arvcamera.h>
 #include <arvsystem.h>
 #include <arvgvinterface.h>
+#include <arvgcboolean.h>
 #include <arvgccommand.h>
 #include <arvgcinteger.h>
 #include <arvgcfloat.h>
@@ -100,6 +101,7 @@ typedef enum {
 } ArvCameraSeries;
 
 typedef struct {
+	char *name;
 	ArvDevice *device;
 	ArvGc *genicam;
 
@@ -109,16 +111,22 @@ typedef struct {
 	gboolean has_gain;
 	gboolean has_exposure_time;
 	gboolean has_acquisition_frame_rate;
+	gboolean has_acquisition_frame_rate_auto;
 	gboolean has_acquisition_frame_rate_enabled;
 
-	GError *error;	/* First error that changed camera status. Cleared after status read.  */
+	GError *init_error;
 } ArvCameraPrivate;
 
-G_DEFINE_TYPE_WITH_CODE (ArvCamera, arv_camera, G_TYPE_OBJECT, G_ADD_PRIVATE (ArvCamera))
+static void arv_camera_initable_iface_init (GInitableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ArvCamera, arv_camera, G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (ArvCamera)
+			 G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, arv_camera_initable_iface_init))
 
 enum
 {
 	PROP_0,
+	PROP_CAMERA_NAME,
 	PROP_CAMERA_DEVICE
 };
 
@@ -127,23 +135,24 @@ enum
  * @camera: a #ArvCamera
  * @callback: (scope call) (allow-none): a frame processing callback
  * @user_data: (closure) (allow-none): user data for @callback
+ * @error: a #GError placeholder, %NULL to ignore
  *
  * Creates a new #ArvStream for video stream handling. See
  * #ArvStreamCallback for details regarding the callback function.
  *
- * Returns: (transfer full): a new #ArvStream.
+ * Returns: (transfer full): a new #ArvStream, to be freed after use with g_object_unref().
  *
  * Since: 0.2.0
  */
 
 ArvStream *
-arv_camera_create_stream (ArvCamera *camera, ArvStreamCallback callback, gpointer user_data)
+arv_camera_create_stream (ArvCamera *camera, ArvStreamCallback callback, gpointer user_data, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
 
-	return arv_device_create_stream (priv->device, callback, user_data);
+	return arv_device_create_stream (priv->device, callback, user_data, error);
 }
 
 /* Device control */
@@ -625,47 +634,49 @@ arv_camera_get_pixel_format_as_string (ArvCamera *camera, GError **error)
 }
 
 /**
- * arv_camera_get_available_pixel_formats:
+ * arv_camera_dup_available_pixel_formats:
  * @camera: a #ArvCamera
  * @n_pixel_formats: (out): number of different pixel formats
  * @error: a #GError placeholder, %NULL to ignore
  *
  * Retrieves the list of all available pixel formats.
  *
- * Returns: (array length=n_pixel_formats) (transfer container): a newly allocated array of #ArvPixelFormat
+ * Returns: (array length=n_pixel_formats) (transfer container): a newly allocated array of #ArvPixelFormat, to be freed after use with
+ * g_free().
  *
  * Since: 0.8.0
  */
 
 gint64 *
-arv_camera_get_available_pixel_formats (ArvCamera *camera, guint *n_pixel_formats, GError **error)
+arv_camera_dup_available_pixel_formats (ArvCamera *camera, guint *n_pixel_formats, GError **error)
 {
-	return arv_camera_get_available_enumerations (camera, "PixelFormat", n_pixel_formats, error);
+	return arv_camera_dup_available_enumerations (camera, "PixelFormat", n_pixel_formats, error);
 }
 
 /**
- * arv_camera_get_available_pixel_formats_as_strings:
+ * arv_camera_dup_available_pixel_formats_as_strings:
  * @camera: a #ArvCamera
  * @n_pixel_formats: (out): number of different pixel formats
  * @error: a #GError placeholder, %NULL to ignore
  *
  * Retrieves the list of all available pixel formats as strings.
  *
- * Returns: (array length=n_pixel_formats) (transfer container): a newly allocated array of strings.
+ * Returns: (array length=n_pixel_formats) (transfer container): a newly allocated array of strings, to be freed after use with
+ * g_free().
  *
  * Since: 0.8.0
  */
 
 const char **
-arv_camera_get_available_pixel_formats_as_strings (ArvCamera *camera, guint *n_pixel_formats, GError **error)
+arv_camera_dup_available_pixel_formats_as_strings (ArvCamera *camera, guint *n_pixel_formats, GError **error)
 {
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
 
-	return arv_camera_get_available_enumerations_as_strings (camera, "PixelFormat", n_pixel_formats, error);
+	return arv_camera_dup_available_enumerations_as_strings (camera, "PixelFormat", n_pixel_formats, error);
 }
 
 /**
- * arv_camera_get_available_pixel_formats_as_display_names:
+ * arv_camera_dup_available_pixel_formats_as_display_names:
  * @camera: a #ArvCamera
  * @n_pixel_formats: (out): number of different pixel formats
  * @error: a #GError placeholder, %NULL to ignore
@@ -673,15 +684,16 @@ arv_camera_get_available_pixel_formats_as_strings (ArvCamera *camera, guint *n_p
  * Retrieves the list of all available pixel formats as display names.
  * In general, these human-readable strings cannot be used as settings.
  *
- * Returns: (array length=n_pixel_formats) (transfer container): a newly allocated array of string constants.
+ * Returns: (array length=n_pixel_formats) (transfer container): a newly allocated array of string constants, to be freed after use with
+ * g_free().
  *
  * Since: 0.8.0
  */
 
 const char **
-arv_camera_get_available_pixel_formats_as_display_names (ArvCamera *camera, guint *n_pixel_formats, GError **error)
+arv_camera_dup_available_pixel_formats_as_display_names (ArvCamera *camera, guint *n_pixel_formats, GError **error)
 {
-	return arv_camera_get_available_enumerations_as_display_names (camera, "PixelFormat", n_pixel_formats, error);
+	return arv_camera_dup_available_enumerations_as_display_names (camera, "PixelFormat", n_pixel_formats, error);
 }
 
 /* Acquisition control */
@@ -752,28 +764,30 @@ arv_camera_acquisition (ArvCamera *camera, guint64 timeout, GError **error)
 {
 	GError *local_error = NULL;
 	ArvStream *stream;
-	ArvBuffer *buffer;
+	ArvBuffer *buffer = NULL;
 	gint payload;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
 
-	stream = arv_camera_create_stream (camera, NULL, NULL);
-	payload = arv_camera_get_payload (camera, &local_error);
-	if (local_error == NULL) {
-		arv_stream_push_buffer (stream,  arv_buffer_new (payload, NULL));
-		arv_camera_set_acquisition_mode (camera, ARV_ACQUISITION_MODE_SINGLE_FRAME, &local_error);
-	}
-	if (local_error == NULL)
-		arv_camera_start_acquisition (camera, &local_error);
-	if (local_error ==NULL) {
-		if (timeout > 0)
-			buffer = arv_stream_timeout_pop_buffer (stream, timeout);
-		else
-			buffer = arv_stream_pop_buffer (stream);
-		arv_camera_stop_acquisition (camera, &local_error);
-	}
+	stream = arv_camera_create_stream (camera, NULL, NULL, &local_error);
+	if (ARV_IS_STREAM(stream)) {
+		payload = arv_camera_get_payload (camera, &local_error);
+		if (local_error == NULL) {
+			arv_stream_push_buffer (stream,  arv_buffer_new (payload, NULL));
+			arv_camera_set_acquisition_mode (camera, ARV_ACQUISITION_MODE_SINGLE_FRAME, &local_error);
+		}
+		if (local_error == NULL)
+			arv_camera_start_acquisition (camera, &local_error);
+		if (local_error == NULL) {
+			if (timeout > 0)
+				buffer = arv_stream_timeout_pop_buffer (stream, timeout);
+			else
+				buffer = arv_stream_pop_buffer (stream);
+			arv_camera_stop_acquisition (camera, &local_error);
+		}
 
-	g_object_unref (stream);
+		g_object_unref (stream);
+	}
 
 	if (local_error != NULL)
 		g_propagate_error (error, local_error);
@@ -897,10 +911,10 @@ arv_camera_get_frame_count_bounds (ArvCamera *camera, gint64 *min, gint64 *max,
  * @frame_rate: frame rate, in Hz
  * @error: a #GError placeholder, %NULL to ignore
  *
- * Configures a fixed frame rate mode. Once acquisition start is triggered, the
- * video stream will be acquired with the given frame rate.
+ * Configures a fixed frame rate mode. Once acquisition start is triggered, the video stream will be acquired with the given frame rate. A
+ * negative or zero @frame_rate value disables the frame rate limit.
  *
- * Since: 0.2.0
+ * Since: 0.8.0
  */
 
 void
@@ -914,8 +928,15 @@ arv_camera_set_frame_rate (ArvCamera *camera, double frame_rate, GError **error)
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
 
-	if (frame_rate <= 0.0)
+	if (frame_rate <= 0.0) {
+		if (arv_camera_is_feature_available (camera, "AcquisitionFrameRateEnable", &local_error)) {
+			if (local_error == NULL)
+				arv_camera_set_boolean (camera, "AcquisitionFrameRateEnable", FALSE, error);
+			else
+				g_propagate_error (error, local_error);
+		}
 		return;
+	}
 
 	arv_camera_get_frame_rate_bounds (camera, &minimum, &maximum, &local_error);
 	if (local_error != NULL) {
@@ -963,7 +984,7 @@ arv_camera_set_frame_rate (ArvCamera *camera, double frame_rate, GError **error)
 					guint n_values;
 					guint i;
 
-					values = arv_camera_get_available_enumerations (camera, "FPS", &n_values, &local_error);
+					values = arv_camera_dup_available_enumerations (camera, "FPS", &n_values, &local_error);
 					for (i = 0; i < n_values && local_error == NULL; i++) {
 						if (values[i] > 0) {
 							double e;
@@ -986,12 +1007,13 @@ arv_camera_set_frame_rate (ArvCamera *camera, double frame_rate, GError **error)
 				arv_camera_set_string (camera, "TriggerMode", "Off", &local_error);
 			if (local_error == NULL) {
 				if (priv->has_acquisition_frame_rate_enabled)
-					arv_camera_set_integer (camera, "AcquisitionFrameRateEnabled", 1, &local_error);
+					arv_camera_set_boolean (camera, "AcquisitionFrameRateEnabled", TRUE, &local_error);
 				else
 					arv_camera_set_boolean (camera, "AcquisitionFrameRateEnable", TRUE, &local_error);
 			}
 			if (local_error == NULL)
-				arv_camera_set_string (camera, "AcquisitionFrameRateAuto", "Off", &local_error);
+				if (priv->has_acquisition_frame_rate_auto)
+					arv_camera_set_string (camera, "AcquisitionFrameRateAuto", "Off", &local_error);
 			if (local_error == NULL)
 				arv_camera_set_float (camera, "AcquisitionFrameRate", frame_rate, &local_error);
 			break;
@@ -1008,6 +1030,12 @@ arv_camera_set_frame_rate (ArvCamera *camera, double frame_rate, GError **error)
 						      priv->has_acquisition_frame_rate ?
 						      "AcquisitionFrameRate":
 						      "AcquisitionFrameRateAbs", frame_rate, &local_error);
+			if (local_error == NULL) {
+				if (arv_camera_is_feature_available (camera, "AcquisitionFrameRateEnable", &local_error)) {
+					if (local_error == NULL)
+						arv_camera_set_boolean (camera, "AcquisitionFrameRateEnable", TRUE, &local_error);
+				}
+			}
 			break;
 	}
 
@@ -1096,7 +1124,7 @@ arv_camera_get_frame_rate_bounds (ArvCamera *camera, double *min, double *max, G
 				guint n_values;
 				guint i;
 
-				values = arv_camera_get_available_enumerations (camera, "FPS", &n_values, &local_error);
+				values = arv_camera_dup_available_enumerations (camera, "FPS", &n_values, &local_error);
 				if (local_error != NULL) {
 					g_propagate_error (error, local_error);
 					return;
@@ -1223,7 +1251,7 @@ arv_camera_get_trigger_source (ArvCamera *camera, GError **error)
 }
 
 /**
- * arv_camera_get_available_trigger_sources:
+ * arv_camera_dup_available_trigger_sources:
  * @camera: a #ArvCamera
  * @n_sources: (out): number of sources
  * @error: a #GError placeholder, %NULL to ignore
@@ -1236,13 +1264,13 @@ arv_camera_get_trigger_source (ArvCamera *camera, GError **error)
  */
 
 const char **
-arv_camera_get_available_trigger_sources (ArvCamera *camera, guint *n_sources, GError **error)
+arv_camera_dup_available_trigger_sources (ArvCamera *camera, guint *n_sources, GError **error)
 {
-	return arv_camera_get_available_enumerations_as_strings (camera, "TriggerSource", n_sources, error);
+	return arv_camera_dup_available_enumerations_as_strings (camera, "TriggerSource", n_sources, error);
 }
 
 /**
- * arv_camera_get_available_triggers:
+ * arv_camera_dup_available_triggers:
  * @camera: a #ArvCamera
  * @n_triggers: (out): number of available triggers
  * @error: a #GError placeholder, %NULL to ignore
@@ -1255,9 +1283,9 @@ arv_camera_get_available_trigger_sources (ArvCamera *camera, guint *n_sources, G
  */
 
 const char **
-arv_camera_get_available_triggers (ArvCamera *camera, guint *n_triggers, GError **error)
+arv_camera_dup_available_triggers (ArvCamera *camera, guint *n_triggers, GError **error)
 {
-	return arv_camera_get_available_enumerations_as_strings (camera, "TriggerSelector", n_triggers, error);
+	return arv_camera_dup_available_enumerations_as_strings (camera, "TriggerSelector", n_triggers, error);
 }
 
 /**
@@ -1280,7 +1308,7 @@ arv_camera_clear_triggers (ArvCamera* camera, GError **error)
 
         g_return_if_fail (ARV_IS_CAMERA (camera));
 
-	triggers = arv_camera_get_available_enumerations_as_strings (camera, "TriggerSelector", &n_triggers, &local_error);
+	triggers = arv_camera_dup_available_triggers (camera, &n_triggers, &local_error);
 
 	for (i = 0; i < n_triggers && local_error == NULL; i++) {
 		arv_camera_set_string (camera, "TriggerSelector", triggers[i], &local_error);
@@ -1288,6 +1316,8 @@ arv_camera_clear_triggers (ArvCamera* camera, GError **error)
 			arv_camera_set_string (camera, "TriggerMode", "Off", &local_error);
 	}
 
+	g_free (triggers);
+
 	if (local_error != NULL)
 		g_propagate_error (error, local_error);
 }
@@ -1786,24 +1816,6 @@ arv_camera_is_binning_available (ArvCamera *camera, GError **error)
 	return horizontal && vertical;
 }
 
-static void
-_update_status (ArvCamera *camera, const GError *error)
-{
-	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-
-	if (error == NULL)
-		return;
-
-	if (priv->error != NULL) {
-		arv_warning_device ("[ArvCamera:_update_status] %s", error->message);
-	} else {
-		arv_warning_device ("[ArvCamera:_update_status] %s (Status changed)", error->message);
-
-		g_clear_error (&priv->error);
-		priv->error = g_error_copy (error);
-	}
-}
-
 /**
  * arv_camera_execute_command:
  * @camera: a #ArvCamera
@@ -1819,16 +1831,10 @@ void
 arv_camera_execute_command (ArvCamera *camera, const char *feature, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
 
-	arv_device_execute_command (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_execute_command (priv->device, feature, error);
 }
 
 /**
@@ -1847,16 +1853,10 @@ void
 arv_camera_set_boolean (ArvCamera *camera, const char *feature, gboolean value, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
 
-	arv_device_set_boolean_feature_value (priv->device, feature, value, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_set_boolean_feature_value (priv->device, feature, value, error);
 }
 
 /**
@@ -1874,21 +1874,28 @@ gboolean
 arv_camera_get_boolean (ArvCamera *camera, const char *feature, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	gboolean value;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), FALSE);
-	g_return_val_if_fail (feature != NULL, FALSE);
 
-	value = arv_device_get_boolean_feature_value (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
+	return arv_device_get_boolean_feature_value (priv->device, feature, error);
+}
 
-		return FALSE;
-	}
+/**
+ * arv_camera_get_boolean_gi: (rename-to arv_camera_get_boolean)
+ * @camera: a #ArvCamera
+ * @feature: feature name
+ * @value: (out): output value
+ * @error: a #GError placeholder, %NULL to ignore
+ *
+ * Since: 0.8.0
+ */
 
-	return value;
+void
+arv_camera_get_boolean_gi (ArvCamera *camera, const char *feature, gboolean *value, GError **error)
+{
+	g_return_if_fail (value != NULL);
+
+	*value = arv_camera_get_boolean (camera, feature, error);
 }
 
 /**
@@ -1907,17 +1914,10 @@ void
 arv_camera_set_string (ArvCamera *camera, const char *feature, const char *value, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
-	g_return_if_fail (value != NULL);
 
-	arv_device_set_string_feature_value (priv->device, feature, value, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_set_string_feature_value (priv->device, feature, value, error);
 }
 
 /**
@@ -1935,21 +1935,10 @@ const char *
 arv_camera_get_string (ArvCamera *camera, const char *feature, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	const char *value;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), FALSE);
-	g_return_val_if_fail (feature != NULL, FALSE);
 
-	value = arv_device_get_string_feature_value (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return NULL;
-	}
-
-	return value;
+	return arv_device_get_string_feature_value (priv->device, feature, error);
 }
 
 /**
@@ -1970,14 +1959,8 @@ arv_camera_set_integer (ArvCamera *camera, const char *feature, gint64 value, GE
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
-	GError *local_error = NULL;
 
-	arv_device_set_integer_feature_value (priv->device, feature, value, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_set_integer_feature_value (priv->device, feature, value, error);
 }
 
 /**
@@ -1995,21 +1978,10 @@ gint64
 arv_camera_get_integer (ArvCamera *camera, const char *feature, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	gint64 value;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), 0);
-	g_return_val_if_fail (feature != NULL, 0);
 
-	value = arv_device_get_integer_feature_value (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return 0;
-	}
-
-	return value;
+	return arv_device_get_integer_feature_value (priv->device, feature, error);
 }
 
 /**
@@ -2029,16 +2001,15 @@ void
 arv_camera_get_integer_bounds (ArvCamera *camera, const char *feature, gint64 *min, gint64 *max, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
+
+	if (min != NULL)
+		*min = G_MININT64;
+	if (max != NULL)
+		*max = G_MAXINT64;
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
 
-	arv_device_get_integer_feature_bounds (priv->device, feature, min, max, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_get_integer_feature_bounds (priv->device, feature, min, max, error);
 }
 
 /**
@@ -2047,7 +2018,7 @@ arv_camera_get_integer_bounds (ArvCamera *camera, const char *feature, gint64 *m
  * @feature: feature name
  * @error: a #GError placeholder, %NULL to ignore
  *
- * Returns: @feature value increment.
+ * Returns: @feature value increment, or 1 on error.
  *
  * Since: 0.8.0
  */
@@ -2056,21 +2027,11 @@ gint64
 arv_camera_get_integer_increment (ArvCamera *camera, const char *feature, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	gint64 value = 1;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), 1);
 	g_return_val_if_fail (feature != NULL, 1);
 
-	value = arv_device_get_integer_feature_increment (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return 0;
-	}
-
-	return value;
+	return arv_device_get_integer_feature_increment (priv->device, feature, error);
 }
 
 static void
@@ -2090,7 +2051,6 @@ arv_camera_get_integer_bounds_as_gint (ArvCamera *camera, const char *feature, g
 
 	arv_device_get_integer_feature_bounds (priv->device, feature, &min64, &max64, &local_error);
 	if (local_error != NULL) {
-		_update_status (camera, local_error);
 		g_propagate_error (error, local_error);
 	} else {
 		if (min != NULL)
@@ -2117,7 +2077,6 @@ arv_camera_get_integer_bounds_as_guint (ArvCamera *camera, const char *feature,
 
 	arv_device_get_integer_feature_bounds (priv->device, feature, &min64, &max64, &local_error);
 	if (local_error != NULL) {
-		_update_status (camera, local_error);
 		g_propagate_error (error, local_error);
 	} else {
 		if (min != NULL)
@@ -2144,7 +2103,6 @@ arv_camera_get_integer_bounds_as_double (ArvCamera *camera, const char *feature,
 
 	arv_device_get_integer_feature_bounds (priv->device, feature, &min64, &max64, &local_error);
 	if (local_error != NULL) {
-		_update_status (camera, local_error);
 		g_propagate_error (error, local_error);
 	} else {
 		if (min != NULL)
@@ -2170,16 +2128,10 @@ void
 arv_camera_set_float (ArvCamera *camera, const char *feature, double value, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
 
-	arv_device_set_float_feature_value (priv->device, feature, value, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_set_float_feature_value (priv->device, feature, value, error);
 }
 
 /**
@@ -2197,21 +2149,10 @@ double
 arv_camera_get_float (ArvCamera *camera, const char *feature, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	double value;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), 0.0);
-	g_return_val_if_fail (feature != NULL, 0.0);
 
-	value = arv_device_get_float_feature_value (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return 0.0;
-	}
-
-	return value;
+	return arv_device_get_float_feature_value (priv->device, feature, error);
 }
 
 /**
@@ -2231,20 +2172,19 @@ void
 arv_camera_get_float_bounds (ArvCamera *camera, const char *feature, double *min, double *max, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
+
+	if (min != NULL)
+		*min = -G_MAXDOUBLE;
+	if (max != NULL)
+		*max = G_MAXDOUBLE;
 
 	g_return_if_fail (ARV_IS_CAMERA (camera));
-	g_return_if_fail (feature != NULL);
 
-	arv_device_get_float_feature_bounds (priv->device, feature, min, max, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_device_get_float_feature_bounds (priv->device, feature, min, max, error);
 }
 
 /**
- * arv_camera_get_available_enumerations:
+ * arv_camera_dup_available_enumerations:
  * @camera: a #ArvCamera
  * @feature: feature name
  * @n_values: placeholder for the number of returned values
@@ -2259,31 +2199,20 @@ arv_camera_get_float_bounds (ArvCamera *camera, const char *feature, double *min
  */
 
 gint64 *
-arv_camera_get_available_enumerations (ArvCamera *camera, const char *feature, guint *n_values, GError **error)
+arv_camera_dup_available_enumerations (ArvCamera *camera, const char *feature, guint *n_values, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	gint64 *values;
 
 	if (n_values != NULL)
 		*n_values = 0;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
-	g_return_val_if_fail (feature != NULL, NULL);
 
-	values = arv_device_get_available_enumeration_feature_values (priv->device, feature, n_values, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return NULL;
-	}
-
-	return values;
+	return arv_device_dup_available_enumeration_feature_values (priv->device, feature, n_values, error);
 }
 
 /**
- * arv_camera_get_available_enumerations_as_strings:
+ * arv_camera_dup_available_enumerations_as_strings:
  * @camera: a #ArvCamera
  * @feature: feature name
  * @n_values: placeholder for the number of returned values
@@ -2298,29 +2227,20 @@ arv_camera_get_available_enumerations (ArvCamera *camera, const char *feature, g
  */
 
 const char **
-arv_camera_get_available_enumerations_as_strings (ArvCamera *camera, const char *feature, guint *n_values, GError **error)
+arv_camera_dup_available_enumerations_as_strings (ArvCamera *camera, const char *feature, guint *n_values, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	const char **strings;
 
 	if (n_values != NULL)
 		*n_values = 0;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
-	g_return_val_if_fail (feature != NULL, NULL);
 
-	strings = arv_device_get_available_enumeration_feature_values_as_strings (priv->device, feature, n_values, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
-
-	return strings;
+	return arv_device_dup_available_enumeration_feature_values_as_strings (priv->device, feature, n_values, error);
 }
 
 /**
- * arv_camera_get_available_enumerations_as_display_names:
+ * arv_camera_dup_available_enumerations_as_display_names:
  * @camera: a #ArvCamera
  * @feature: feature name
  * @n_values: placeholder for the number of returned values
@@ -2335,25 +2255,16 @@ arv_camera_get_available_enumerations_as_strings (ArvCamera *camera, const char
  */
 
 const char **
-arv_camera_get_available_enumerations_as_display_names (ArvCamera *camera, const char *feature, guint *n_values, GError **error)
+arv_camera_dup_available_enumerations_as_display_names (ArvCamera *camera, const char *feature, guint *n_values, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	const char **strings;
 
 	if (n_values != NULL)
 		*n_values = 0;
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
-	g_return_val_if_fail (feature != NULL, NULL);
 
-	strings = arv_device_get_available_enumeration_feature_values_as_display_names (priv->device, feature, n_values, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
-
-	return strings;
+	return arv_device_dup_available_enumeration_feature_values_as_display_names (priv->device, feature, n_values, error);
 }
 
 /**
@@ -2369,39 +2280,12 @@ arv_camera_get_available_enumerations_as_display_names (ArvCamera *camera, const
 
 gboolean
 arv_camera_is_feature_available (ArvCamera *camera, const char *feature, GError **error)
-{
-	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	gboolean available = FALSE;
-
-	g_return_val_if_fail (ARV_IS_CAMERA (camera), FALSE);
-	g_return_val_if_fail (feature != NULL, FALSE);
-
-	available = arv_device_is_feature_available (priv->device, feature, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return FALSE;
-	}
-
-	return available;
-}
-
-gboolean
-arv_camera_check_status (ArvCamera *camera, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
 
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), FALSE);
 
-	if (priv->error == NULL)
-		return TRUE;
-
-	g_propagate_error (error, priv->error);
-	priv->error = NULL;
-
-	return FALSE;
+	return arv_device_is_feature_available (priv->device, feature, error);
 }
 
 /**
@@ -2470,7 +2354,7 @@ arv_camera_gv_select_stream_channel (ArvCamera *camera, gint channel_id, GError
  *
  * Returns: The current stream channel id.
  *
- * Since: 0.4.0
+ * Since: 0.8.0
  */
 
 int
@@ -2574,7 +2458,6 @@ arv_camera_gv_get_packet_delay (ArvCamera *camera, GError **error)
 void
 arv_camera_gv_set_packet_size (ArvCamera *camera, gint packet_size, GError **error)
 {
-	GError *local_error = NULL;
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
 
 	if (packet_size <= 0)
@@ -2582,11 +2465,7 @@ arv_camera_gv_set_packet_size (ArvCamera *camera, gint packet_size, GError **err
 
 	g_return_if_fail (arv_camera_is_gv_device (camera));
 
-	arv_gv_device_set_packet_size (ARV_GV_DEVICE (priv->device), packet_size, &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-	}
+	arv_gv_device_set_packet_size (ARV_GV_DEVICE (priv->device), packet_size, error);
 }
 
 /**
@@ -2603,20 +2482,10 @@ guint
 arv_camera_gv_get_packet_size (ArvCamera *camera, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	guint value;
 
 	g_return_val_if_fail (arv_camera_is_gv_device (camera), 0);
 
-	value = arv_gv_device_get_packet_size (ARV_GV_DEVICE (priv->device), &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return 0;
-	}
-
-	return value;
+	return arv_gv_device_get_packet_size (ARV_GV_DEVICE (priv->device), error);
 }
 
 /**
@@ -2638,20 +2507,10 @@ guint
 arv_camera_gv_auto_packet_size (ArvCamera *camera, GError **error)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (camera);
-	GError *local_error = NULL;
-	guint value;
 
 	g_return_val_if_fail (arv_camera_is_gv_device (camera), 0);
 
-	value = arv_gv_device_auto_packet_size (ARV_GV_DEVICE (priv->device), &local_error);
-	if (local_error != NULL) {
-		_update_status (camera, local_error);
-		g_propagate_error (error, local_error);
-
-		return 0;
-	}
-
-	return value;
+	return arv_gv_device_auto_packet_size (ARV_GV_DEVICE (priv->device), error);
 }
 
 /**
@@ -2917,7 +2776,7 @@ arv_camera_set_chunks (ArvCamera *camera, const char *chunk_list, GError **error
 		return;
 	}
 
-	available_chunks = arv_camera_get_available_enumerations_as_strings (camera, "ChunkSelector", &n_values, &local_error);
+	available_chunks = arv_camera_dup_available_enumerations_as_strings (camera, "ChunkSelector", &n_values, &local_error);
 	for (i = 0; i < n_values && local_error == NULL; i++) {
 		arv_camera_set_chunk_state (camera, available_chunks[i], FALSE, &local_error);
 	}
@@ -2972,34 +2831,25 @@ arv_camera_create_chunk_parser (ArvCamera *camera)
 /**
  * arv_camera_new:
  * @name: (allow-none): name of the camera.
+ * @error: a #GError placeholder, %NULL to ignore
  *
  * Creates a new #ArvCamera. If @name is null, it will instantiate the
  * first available camera.
  *
  * Returns: a new #ArvCamera.
  *
- * Since: 0.2.0
+ * Since: 0.8.0
  */
 
 ArvCamera *
-arv_camera_new (const char *name)
+arv_camera_new (const char *name, GError **error)
 {
-	ArvCamera *camera;
-	ArvDevice *device;
-
-	device = arv_open_device (name);
-
-	if (!ARV_IS_DEVICE (device))
-		return NULL;
-
-	camera = g_object_new (ARV_TYPE_CAMERA, "device", device, NULL);
-
 	/* if you need to apply or test for fixups based on the camera model
 	   please do so in arv_camera_constructor and not here, as this breaks
 	   objects created with g_object_new, which includes but is not limited to
 	   introspection users */
 
-	return camera;
+	return g_initable_new (ARV_TYPE_CAMERA, NULL, error, "name", name, NULL);
 }
 
 static void
@@ -3012,33 +2862,44 @@ arv_camera_finalize (GObject *object)
 {
 	ArvCameraPrivate *priv = arv_camera_get_instance_private (ARV_CAMERA (object));
 
+	g_clear_pointer (&priv->name, g_free);
 	g_clear_object (&priv->device);
-	g_clear_error (&priv->error);
+	g_clear_error (&priv->init_error);
 
 	G_OBJECT_CLASS (arv_camera_parent_class)->finalize (object);
 }
 
-static GObject *
-arv_camera_constructor (GType gtype, guint n_properties, GObjectConstructParam *properties)
+static void
+arv_camera_constructed (GObject *object)
 {
+	ArvCamera *camera = ARV_CAMERA (object);
 	ArvCameraPrivate *priv;
-	GObject *object;
-	ArvCamera *camera;
 	ArvCameraVendor vendor;
 	ArvCameraSeries series;
 	const char *vendor_name;
 	const char *model_name;
+	GError *error = NULL;
 
-	object = G_OBJECT_CLASS (arv_camera_parent_class)->constructor (gtype, n_properties, properties);
-
-	camera = ARV_CAMERA (object);
 	priv = arv_camera_get_instance_private (camera);
 
 	if (!priv->device)
-		priv->device = arv_open_device (NULL);
+		priv->device = arv_open_device (priv->name, &error);
 
-	if (!ARV_IS_DEVICE (priv->device))
-		return NULL;
+	if (!ARV_IS_DEVICE (priv->device)) {
+		if (error == NULL) {
+			if (priv->name != NULL)
+				error = g_error_new (ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_NOT_FOUND,
+						     "Device '%s' not found", priv->name);
+			else
+				error = g_error_new (ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_NOT_FOUND,
+						     "No supported deice found");
+		}
+
+		g_clear_error (&priv->init_error);
+		priv->init_error = error;
+
+		return;
+	}
 
 	priv->genicam = arv_device_get_genicam (priv->device);
 
@@ -3086,10 +2947,10 @@ arv_camera_constructor (GType gtype, guint n_properties, GObjectConstructParam *
 	priv->has_exposure_time = ARV_IS_GC_FLOAT (arv_device_get_feature (priv->device, "ExposureTime"));
 	priv->has_acquisition_frame_rate = ARV_IS_GC_FLOAT (arv_device_get_feature (priv->device,
 										    "AcquisitionFrameRate"));
-	priv->has_acquisition_frame_rate_enabled = ARV_IS_GC_INTEGER (arv_device_get_feature (priv->device,
+	priv->has_acquisition_frame_rate_auto = ARV_IS_GC_STRING (arv_device_get_feature (priv->device,
+											  "AcquisitionFrameRateAuto"));
+	priv->has_acquisition_frame_rate_enabled = ARV_IS_GC_BOOLEAN (arv_device_get_feature (priv->device,
 											      "AcquisitionFrameRateEnabled"));
-
-    return object;
 }
 
 static void
@@ -3102,6 +2963,10 @@ arv_camera_set_property (GObject *object, guint prop_id, const GValue *value, GP
 		case PROP_CAMERA_DEVICE:
 			priv->device = g_value_get_object (value);
 			break;
+		case PROP_CAMERA_NAME:
+			g_free (priv->name);
+			priv->name = g_value_dup_string (value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -3130,15 +2995,58 @@ arv_camera_class_init (ArvCameraClass *camera_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (camera_class);
 
 	object_class->finalize = arv_camera_finalize;
-	object_class->constructor = arv_camera_constructor;
+	object_class->constructed = arv_camera_constructed;
 	object_class->set_property = arv_camera_set_property;
 	object_class->get_property = arv_camera_get_property;
 
-	g_object_class_install_property (object_class,
-					 PROP_CAMERA_DEVICE,
-					 g_param_spec_object ("device",
-							      "device",
-							      "the device associated with this camera",
-							      ARV_TYPE_DEVICE,
-							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property
+		(object_class,
+		 PROP_CAMERA_NAME,
+		 g_param_spec_string ("name",
+				      "Camera name",
+				      "The camera name",
+				      NULL,
+				      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property
+		(object_class,
+		 PROP_CAMERA_DEVICE,
+		 g_param_spec_object ("device",
+				      "Device",
+				      "The device associated with this camera",
+				      ARV_TYPE_DEVICE,
+				      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
+
+static gboolean
+arv_camera_initable_init (GInitable     *initable,
+			  GCancellable  *cancellable,
+			  GError       **error)
+{
+	ArvCamera *self = ARV_CAMERA (initable);
+	ArvCameraPrivate *priv = arv_camera_get_instance_private (ARV_CAMERA (initable));
+
+	g_return_val_if_fail (ARV_IS_CAMERA (self), FALSE);
+
+	if (cancellable != NULL)
+	{
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+				     "Cancellable initialization not supported");
+		return FALSE;
+	}
+
+	if (priv->init_error != NULL) {
+		if (error != NULL)
+			*error = g_error_copy (priv->init_error);
+		 return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static void
+arv_camera_initable_iface_init (GInitableIface *iface)
+{
+	iface->init = arv_camera_initable_init;
+}
+
