@@ -35,7 +35,28 @@
 #include <arvstr.h>
 #include <string.h>
 
-static GObjectClass *parent_class = NULL;
+struct _ArvGcFloatNode {
+	ArvGcFeatureNode	node;
+
+	ArvGcPropertyNode *value;
+	ArvGcPropertyNode *minimum;
+	ArvGcPropertyNode *maximum;
+	ArvGcPropertyNode *increment;
+	ArvGcPropertyNode *unit;
+
+	ArvGcPropertyNode *index;
+	GSList *value_indexed_nodes;
+	ArvGcPropertyNode *value_default;
+};
+
+struct _ArvGcFloatNodeClass {
+	ArvGcFeatureNodeClass parent_class;
+};
+
+static void arv_gc_float_node_float_interface_init (ArvGcFloatInterface *interface);
+
+G_DEFINE_TYPE_WITH_CODE (ArvGcFloatNode, arv_gc_float_node, ARV_TYPE_GC_FEATURE_NODE,
+			 G_IMPLEMENT_INTERFACE (ARV_TYPE_GC_FLOAT, arv_gc_float_node_float_interface_init))
 
 /* ArvDomNode implementation */
 
@@ -85,7 +106,7 @@ arv_gc_float_node_post_new_child (ArvDomNode *self, ArvDomNode *child)
 				node->value_default = property_node;
 				break;
 			default:
-				ARV_DOM_NODE_CLASS (parent_class)->post_new_child (self, child);
+				ARV_DOM_NODE_CLASS (arv_gc_float_node_parent_class)->post_new_child (self, child);
 				break;
 		}
 	}
@@ -150,7 +171,7 @@ arv_gc_float_node_finalize (GObject *object)
 {
 	ArvGcFloatNode *gc_float_node = ARV_GC_FLOAT_NODE (object);
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (arv_gc_float_node_parent_class)->finalize (object);
 
 	g_slist_free (gc_float_node->value_indexed_nodes);
 }
@@ -161,8 +182,6 @@ arv_gc_float_node_class_init (ArvGcFloatNodeClass *this_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (this_class);
 	ArvDomNodeClass *dom_node_class = ARV_DOM_NODE_CLASS (this_class);
 
-	parent_class = g_type_class_peek_parent (this_class);
-
 	object_class->finalize = arv_gc_float_node_finalize;
 	dom_node_class->get_node_name = arv_gc_float_node_get_node_name;
 	dom_node_class->post_new_child = arv_gc_float_node_post_new_child;
@@ -383,6 +402,3 @@ arv_gc_float_node_float_interface_init (ArvGcFloatInterface *interface)
 	interface->impose_min = arv_gc_float_node_impose_min;
 	interface->impose_max = arv_gc_float_node_impose_max;
 }
-
-G_DEFINE_TYPE_WITH_CODE (ArvGcFloatNode, arv_gc_float_node, ARV_TYPE_GC_FEATURE_NODE,
-			 G_IMPLEMENT_INTERFACE (ARV_TYPE_GC_FLOAT, arv_gc_float_node_float_interface_init))
