@@ -49,18 +49,7 @@ G_BEGIN_DECLS
 
 #define ARV_GV_DEVICE_BUFFER_SIZE	1024
 
-typedef struct _ArvGvDeviceClass ArvGvDeviceClass;
-typedef struct _ArvGvDevicePrivate ArvGvDevicePrivate;
-
-struct _ArvGvDevice {
-	ArvDevice device;
-
-	ArvGvDevicePrivate *priv;
-};
-
-struct _ArvGvDeviceClass {
-	ArvDeviceClass parent_class;
-};
+GRegex * 		arv_gv_device_get_url_regex 			(void);
 
 G_END_DECLS
 
