@@ -40,7 +40,6 @@
 #include <sys/socket.h>
 #include <stdio.h>
 #include <errno.h>
-#include <arvwakeupprivate.h>
 
 #if ARAVIS_HAS_PACKET_SOCKET
 #include <ifaddrs.h>
@@ -83,8 +82,6 @@ typedef struct {
 
 struct _ArvGvStream {
 	ArvStream	stream;
-
-	ArvGvStreamPrivate *priv;
 };
 
 struct _ArvGvStreamClass {
@@ -115,11 +112,13 @@ typedef struct {
 } ArvGvStreamFrameData;
 
 struct _ArvGvStreamThreadData {
+	GCancellable *cancellable;
+
 	ArvGvDevice *gv_device;
 	ArvStream *stream;
 
 	ArvStreamCallback callback;
-	void *user_data;
+	void *callback_data;
 
 	GSocket *socket;
 	GInetAddress *interface_address;
@@ -137,9 +136,6 @@ struct _ArvGvStreamThreadData {
 	guint64 timestamp_tick_frequency;
 	guint data_size;
 
-	gboolean exit_thread;
-	ArvWakeup *wakeup;
-
 	guint16 packet_id;
 
 	GSList *frames;
@@ -405,7 +401,7 @@ _find_frame_data (ArvGvStreamThreadData *thread_data,
 
 	if (thread_data->callback != NULL &&
 	    frame->buffer != NULL)
-		thread_data->callback (thread_data->user_data,
+		thread_data->callback (thread_data->callback_data,
 				       ARV_STREAM_CALLBACK_TYPE_START_BUFFER,
 				       NULL);
 
@@ -535,7 +531,7 @@ _close_frame (ArvGvStreamThreadData *thread_data, ArvGvStreamFrameData *frame)
 
 	arv_stream_push_output_buffer (thread_data->stream, frame->buffer);
 	if (thread_data->callback != NULL)
-		thread_data->callback (thread_data->user_data,
+		thread_data->callback (thread_data->callback_data,
 				       ARV_STREAM_CALLBACK_TYPE_BUFFER_DONE,
 				       frame->buffer);
 
@@ -683,13 +679,14 @@ _process_packet (ArvGvStreamThreadData *thread_data, const ArvGvspPacket *packet
 			frame->error_packet_received = TRUE;
 
 			thread_data->n_error_packets++;
+		} else if (packet_id < frame->n_packets &&
+		           frame->packet_data[packet_id].received) {
+			/* Ignore duplicate packet */
+			thread_data->n_duplicated_packets++;
+			arv_gvsp_packet_debug (packet, packet_size, ARV_DEBUG_LEVEL_LOG);
 		} else {
-			/* Check for duplicated packets */
 			if (packet_id < frame->n_packets) {
-				if (frame->packet_data[packet_id].received)
-					thread_data->n_duplicated_packets++;
-				else
-					frame->packet_data[packet_id].received = TRUE;
+				frame->packet_data[packet_id].received = TRUE;
 			}
 
 			/* Keep track of last packet of a continuous block starting from packet 0 */
@@ -740,40 +737,44 @@ _loop (ArvGvStreamThreadData *thread_data)
 	poll_fd[0].events =  G_IO_IN;
 	poll_fd[0].revents = 0;
 
-	arv_wakeup_get_pollfd (thread_data->wakeup, &poll_fd[1]);
-
 	packet = g_malloc0 (ARV_GV_STREAM_INCOMING_BUFFER_SIZE);
 
-	do {
-		int n_events;
-		int errsv;
-
-		if (thread_data->frames != NULL)
-			timeout_ms = thread_data->packet_timeout_us / 1000;
-		else
-			timeout_ms = ARV_GV_STREAM_POLL_TIMEOUT_US / 1000;
-
+	if (g_cancellable_make_pollfd (thread_data->cancellable, &poll_fd[1])) {
 		do {
-			poll_fd[0].revents = 0;
+			int n_events;
+			int errsv;
 
-			n_events = g_poll (poll_fd, 2, timeout_ms);
-			errsv = errno;
+			if (thread_data->frames != NULL)
+				timeout_ms = thread_data->packet_timeout_us / 1000;
+			else
+				timeout_ms = ARV_GV_STREAM_POLL_TIMEOUT_US / 1000;
 
-		} while (n_events < 0 && errsv == EINTR);
+			do {
+				poll_fd[0].revents = 0;
 
-		time_us = g_get_monotonic_time ();
+				n_events = g_poll (poll_fd, 2, timeout_ms);
+				errsv = errno;
 
-		if (poll_fd[0].revents != 0) {
-			read_count = g_socket_receive (thread_data->socket, (char *) packet,
-						       ARV_GV_STREAM_INCOMING_BUFFER_SIZE, NULL, NULL);
+			} while (n_events < 0 && errsv == EINTR);
 
-			frame = _process_packet (thread_data, packet, read_count, time_us);
-		} else
-			frame = NULL;
+			time_us = g_get_monotonic_time ();
 
-		_check_frame_completion (thread_data, time_us, frame);
+			if (poll_fd[0].revents != 0) {
+				read_count = g_socket_receive (thread_data->socket, (char *) packet,
+							       ARV_GV_STREAM_INCOMING_BUFFER_SIZE, NULL, NULL);
 
-	} while (!g_atomic_int_get (&thread_data->exit_thread));
+				frame = _process_packet (thread_data, packet, read_count, time_us);
+			} else
+				frame = NULL;
+
+			_check_frame_completion (thread_data, time_us, frame);
+
+		} while (!g_cancellable_is_cancelled (thread_data->cancellable));
+
+		g_cancellable_release_fd (thread_data->cancellable);
+	} else {
+		g_error ("[ArvGvStream::_loop] Failed to create cancellable fd");
+	}
 
 	g_free (packet);
 }
@@ -916,53 +917,57 @@ _ring_buffer_loop (ArvGvStreamThreadData *thread_data)
 	poll_fd[0].events =  G_IO_IN;
 	poll_fd[0].revents = 0;
 
-	arv_wakeup_get_pollfd (thread_data->wakeup, &poll_fd[1]);
+	if (g_cancellable_make_pollfd (thread_data->cancellable, &poll_fd[1])) {
+		block_id = 0;
+		do {
+			ArvGvStreamBlockDescriptor *descriptor;
+			guint64 time_us;
 
-	block_id = 0;
-	do {
-		ArvGvStreamBlockDescriptor *descriptor;
-		guint64 time_us;
+			time_us = g_get_monotonic_time ();
 
-		time_us = g_get_monotonic_time ();
+			descriptor = (void *) (buffer + block_id * req.tp_block_size);
+			if ((descriptor->h1.block_status & TP_STATUS_USER) == 0) {
+				int n_events;
+				int errsv;
 
-		descriptor = (void *) (buffer + block_id * req.tp_block_size);
-		if ((descriptor->h1.block_status & TP_STATUS_USER) == 0) {
-			int n_events;
-			int errsv;
+				_check_frame_completion (thread_data, time_us, NULL);
 
-			_check_frame_completion (thread_data, time_us, NULL);
+				do {
+					n_events = g_poll (poll_fd, 2, 100);
+					errsv = errno;
+				} while (n_events < 0 && errsv == EINTR);
+			} else {
+				ArvGvStreamFrameData *frame;
+				const struct tpacket3_hdr *header;
+				unsigned i;
 
-			do {
-				n_events = g_poll (poll_fd, 2, 100);
-				errsv = errno;
-			} while (n_events < 0 && errsv == EINTR);
-		} else {
-			ArvGvStreamFrameData *frame;
-			const struct tpacket3_hdr *header;
-			unsigned i;
+				header = (void *) (((char *) descriptor) + descriptor->h1.offset_to_first_pkt);
 
-			header = (void *) (((char *) descriptor) + descriptor->h1.offset_to_first_pkt);
+				for (i = 0; i < descriptor->h1.num_pkts; i++) {
+					const struct iphdr *ip;
+					const ArvGvspPacket *packet;
+					size_t size;
 
-			for (i = 0; i < descriptor->h1.num_pkts; i++) {
-				const struct iphdr *ip;
-				const ArvGvspPacket *packet;
-				size_t size;
+					ip = (void *) (((char *) header) + header->tp_mac + ETH_HLEN);
+					packet = (void *) (((char *) ip) + sizeof (struct iphdr) + sizeof (struct udphdr));
+					size = g_ntohs (ip->tot_len) -  sizeof (struct iphdr) - sizeof (struct udphdr);
 
-				ip = (void *) (((char *) header) + header->tp_mac + ETH_HLEN);
-				packet = (void *) (((char *) ip) + sizeof (struct iphdr) + sizeof (struct udphdr));
-				size = g_ntohs (ip->tot_len) -  sizeof (struct iphdr) - sizeof (struct udphdr);
+					frame = _process_packet (thread_data, packet, size, time_us);
 
-				frame = _process_packet (thread_data, packet, size, time_us);
+					_check_frame_completion (thread_data, time_us, frame);
 
-				_check_frame_completion (thread_data, time_us, frame);
+					header = (void *) (((char *) header) + header->tp_next_offset);
+				}
 
-				header = (void *) (((char *) header) + header->tp_next_offset);
+				descriptor->h1.block_status = TP_STATUS_KERNEL;
+				block_id = (block_id + 1) % req.tp_block_nr;
 			}
+		} while (!g_cancellable_is_cancelled (thread_data->cancellable));
 
-			descriptor->h1.block_status = TP_STATUS_KERNEL;
-			block_id = (block_id + 1) % req.tp_block_nr;
-		}
-	} while (!g_atomic_int_get (&thread_data->exit_thread));
+		g_cancellable_release_fd (thread_data->cancellable);
+	} else {
+		g_error ("[ArvGvStream::_ring_buffer_loop] Failed to create cancellable fd");
+	}
 
 bind_error:
 	munmap (buffer, req.tp_block_size * req.tp_block_nr);
@@ -991,7 +996,7 @@ arv_gv_stream_thread (void *data)
 				 thread_data->frame_retention_us / 1000.0);
 
 	if (thread_data->callback != NULL)
-		thread_data->callback (thread_data->user_data, ARV_STREAM_CALLBACK_TYPE_INIT, NULL);
+		thread_data->callback (thread_data->callback_data, ARV_STREAM_CALLBACK_TYPE_INIT, NULL);
 
 #if ARAVIS_HAS_PACKET_SOCKET
 	if (thread_data->use_packet_socket && (fd = socket (PF_PACKET, SOCK_RAW, g_htons (ETH_P_ALL))) >= 0) {
@@ -1004,7 +1009,7 @@ arv_gv_stream_thread (void *data)
 	_flush_frames (thread_data);
 
 	if (thread_data->callback != NULL)
-		thread_data->callback (thread_data->user_data, ARV_STREAM_CALLBACK_TYPE_EXIT, NULL);
+		thread_data->callback (thread_data->callback_data, ARV_STREAM_CALLBACK_TYPE_EXIT, NULL);
 
 	return NULL;
 }
@@ -1014,71 +1019,82 @@ arv_gv_stream_thread (void *data)
 guint16
 arv_gv_stream_get_port (ArvGvStream *gv_stream)
 {
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (gv_stream);
+
 	g_return_val_if_fail (ARV_IS_GV_STREAM (gv_stream), 0);
 
-	return gv_stream->priv->thread_data->stream_port;
+	return priv->thread_data->stream_port;
 }
 
 static void
 arv_gv_stream_start_thread (ArvStream *stream)
 {
-	ArvGvStream *gv_stream = ARV_GV_STREAM (stream);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (stream));
+	ArvGvStreamThreadData *thread_data;
 
-	g_return_if_fail (gv_stream->priv->thread == NULL);
-	g_return_if_fail (gv_stream->priv->thread_data != NULL);
+	g_return_if_fail (priv->thread == NULL);
+	g_return_if_fail (priv->thread_data != NULL);
 
-	gv_stream->priv->thread_data->exit_thread = FALSE;
+	thread_data = priv->thread_data;
 
-	gv_stream->priv->thread = g_thread_new ("arv_gv_stream", arv_gv_stream_thread, gv_stream->priv->thread_data);
+	thread_data->cancellable = g_cancellable_new ();
+	priv->thread = g_thread_new ("arv_gv_stream", arv_gv_stream_thread, priv->thread_data);
 }
 
 static void
 arv_gv_stream_stop_thread (ArvStream *stream)
 {
-	ArvGvStream *gv_stream = ARV_GV_STREAM (stream);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (stream));
 	ArvGvStreamThreadData *thread_data;
 
-	g_return_if_fail (gv_stream->priv->thread != NULL);
-	g_return_if_fail (gv_stream->priv->thread_data != NULL);
+	g_return_if_fail (priv->thread != NULL);
+	g_return_if_fail (priv->thread_data != NULL);
 
-	thread_data = gv_stream->priv->thread_data;
+	thread_data = priv->thread_data;
 
-	g_atomic_int_set (&thread_data->exit_thread, TRUE);
-	arv_wakeup_signal (thread_data->wakeup);
-	g_thread_join (gv_stream->priv->thread);
+	g_cancellable_cancel (thread_data->cancellable);
+	g_thread_join (priv->thread);
+	g_clear_object (&thread_data->cancellable);
 
-	gv_stream->priv->thread = NULL;
+	priv->thread = NULL;
 }
 
 /**
  * arv_gv_stream_new: (skip)
  * @gv_device: a #ArvGvDevice
- * @interface_address: inet interface address for gvsp
- * @device_address: inet device address for gvsp
  * @callback: (scope call): processing callback
- * @user_data: (closure): user data for @callback
+ * @callback_data: (closure): user data for @callback
  *
  * Return value: (transfer full): a new #ArvStream.
  */
 
 ArvStream *
-arv_gv_stream_new (ArvGvDevice *gv_device,
-		   GInetAddress *interface_address,
-		   GInetAddress *device_address,
-		   ArvStreamCallback callback, void *user_data)
+arv_gv_stream_new (ArvGvDevice *gv_device, ArvStreamCallback callback, void *callback_data, GError **error)
 {
+	return g_initable_new (ARV_TYPE_GV_STREAM, NULL, error,
+			       "device", gv_device,
+			       "callback", callback,
+			       "callback-data", callback_data,
+			       NULL);
+}
+
+void
+arv_gv_stream_constructed (GObject *object)
+{
+	ArvStream *stream = ARV_STREAM (object);
+	ArvGvStream *gv_stream = ARV_GV_STREAM (object);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (stream));
 	ArvGvStreamThreadData *thread_data;
-	ArvGvStream *gv_stream;
-	ArvStream *stream;
 	ArvGvStreamOption options;
+	ArvGvDevice *gv_device = NULL;
+	GInetAddress *interface_address;
+	GInetAddress *device_address;
 	guint64 timestamp_tick_frequency;
 	const guint8 *address_bytes;
 	GInetSocketAddress *local_address;
 	guint packet_size;
 
-	g_return_val_if_fail (ARV_IS_GV_DEVICE (gv_device), NULL);
-	g_return_val_if_fail (G_IS_INET_ADDRESS (interface_address), NULL);
-	g_return_val_if_fail (G_IS_INET_ADDRESS (device_address), NULL);
+	g_object_get (object, "device", &gv_device, NULL);
 
 	timestamp_tick_frequency = arv_gv_device_get_timestamp_tick_frequency (gv_device, NULL);
 	options = arv_gv_device_get_stream_options (gv_device);
@@ -1093,17 +1109,22 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	packet_size = arv_gv_device_get_packet_size (gv_device, NULL);
 	arv_debug_device ("[GvStream::stream_new] Packet size = %d byte(s)", packet_size);
 
-	g_return_val_if_fail (packet_size > ARV_GVSP_PACKET_PROTOCOL_OVERHEAD, NULL);
-
-	gv_stream = g_object_new (ARV_TYPE_GV_STREAM, NULL);
-
-	stream = ARV_STREAM (gv_stream);
+	if (packet_size <= ARV_GVSP_PACKET_PROTOCOL_OVERHEAD) {
+		arv_stream_take_init_error (stream, g_error_new (ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_PROTOCOL_ERROR,
+								 "Invalid packet size (%d byte(s))", packet_size));
+		return;
+	}
 
 	thread_data = g_new (ArvGvStreamThreadData, 1);
-	thread_data->gv_device = g_object_ref (gv_device);
+
 	thread_data->stream = stream;
-	thread_data->callback = callback;
-	thread_data->user_data = user_data;
+	thread_data->gv_device = gv_device;
+
+	g_object_get (object,
+		      "callback", &thread_data->callback,
+		      "callback-data", &thread_data->callback_data,
+		      NULL);
+
 	thread_data->packet_resend = ARV_GV_STREAM_PACKET_RESEND_ALWAYS;
 	thread_data->packet_request_ratio = ARV_GV_STREAM_PACKET_REQUEST_RATIO_DEFAULT;
 	thread_data->packet_timeout_us = ARV_GV_STREAM_PACKET_TIMEOUT_US_DEFAULT;
@@ -1111,7 +1132,6 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	thread_data->timestamp_tick_frequency = timestamp_tick_frequency;
 	thread_data->data_size = packet_size - ARV_GVSP_PACKET_PROTOCOL_OVERHEAD;
 	thread_data->use_packet_socket = (options & ARV_GV_STREAM_OPTION_PACKET_SOCKET_DISABLED) == 0;
-	thread_data->exit_thread = FALSE;
 
 	thread_data->packet_id = 65300;
 	thread_data->last_frame_id = 0;
@@ -1142,7 +1162,10 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	thread_data->socket_buffer_size = 0;
 	thread_data->current_socket_buffer_size = 0;
 
-	gv_stream->priv->thread_data = thread_data;
+	priv->thread_data = thread_data;
+
+	interface_address = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (arv_gv_device_get_interface_address (gv_device)));
+	device_address = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (arv_gv_device_get_device_address (gv_device)));
 
 	thread_data->socket = g_socket_new (G_SOCKET_FAMILY_IPV4,
 					  G_SOCKET_TYPE_DATAGRAM,
@@ -1162,14 +1185,10 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	arv_device_set_integer_feature_value (ARV_DEVICE (gv_device), "GevSCPHostPort", thread_data->stream_port, NULL);
 	thread_data->source_stream_port = arv_device_get_integer_feature_value (ARV_DEVICE (gv_device), "GevSCSP", NULL);
 
-	thread_data->wakeup = arv_wakeup_new();
-
 	arv_debug_stream ("[GvStream::stream_new] Destination stream port = %d", thread_data->stream_port);
 	arv_debug_stream ("[GvStream::stream_new] Source stream port = %d", thread_data->source_stream_port);
 
 	arv_gv_stream_start_thread (ARV_STREAM (gv_stream));
-
-	return ARV_STREAM (gv_stream);
 }
 
 /* ArvStream implementation */
@@ -1187,11 +1206,12 @@ arv_gv_stream_get_statistics (ArvGvStream *gv_stream,
 			      guint64 *n_missing_packets)
 
 {
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (gv_stream);
 	ArvGvStreamThreadData *thread_data;
 
 	g_return_if_fail (ARV_IS_GV_STREAM (gv_stream));
 
-	thread_data = gv_stream->priv->thread_data;
+	thread_data = priv->thread_data;
 
 	if (n_resent_packets != NULL)
 		*n_resent_packets = thread_data->n_resent_packets;
@@ -1205,10 +1225,10 @@ _get_statistics (ArvStream *stream,
 		 guint64 *n_failures,
 		 guint64 *n_underruns)
 {
-	ArvGvStream *gv_stream = ARV_GV_STREAM (stream);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (stream));
 	ArvGvStreamThreadData *thread_data;
 
-	thread_data = gv_stream->priv->thread_data;
+	thread_data = priv->thread_data;
 
 	*n_completed_buffers = thread_data->n_completed_buffers;
 	*n_failures = thread_data->n_failures;
@@ -1219,10 +1239,10 @@ static void
 arv_gv_stream_set_property (GObject * object, guint prop_id,
 			    const GValue * value, GParamSpec * pspec)
 {
-	ArvGvStream *gv_stream = ARV_GV_STREAM (object);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (object));
 	ArvGvStreamThreadData *thread_data;
 
-	thread_data = gv_stream->priv->thread_data;
+	thread_data = priv->thread_data;
 
 	switch (prop_id) {
 		case ARV_GV_STREAM_PROPERTY_SOCKET_BUFFER:
@@ -1253,10 +1273,10 @@ static void
 arv_gv_stream_get_property (GObject * object, guint prop_id,
 			    GValue * value, GParamSpec * pspec)
 {
-	ArvGvStream *gv_stream = ARV_GV_STREAM (object);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (object));
 	ArvGvStreamThreadData *thread_data;
 
-	thread_data = gv_stream->priv->thread_data;
+	thread_data = priv->thread_data;
 
 	switch (prop_id) {
 		case ARV_GV_STREAM_PROPERTY_SOCKET_BUFFER:
@@ -1286,21 +1306,20 @@ arv_gv_stream_get_property (GObject * object, guint prop_id,
 static void
 arv_gv_stream_init (ArvGvStream *gv_stream)
 {
-	gv_stream->priv = arv_gv_stream_get_instance_private (gv_stream);
 }
 
 static void
 arv_gv_stream_finalize (GObject *object)
 {
-	ArvGvStream *gv_stream = ARV_GV_STREAM (object);
+	ArvGvStreamPrivate *priv = arv_gv_stream_get_instance_private (ARV_GV_STREAM (object));
 
-	arv_gv_stream_stop_thread (ARV_STREAM (gv_stream));
+	arv_gv_stream_stop_thread (ARV_STREAM (object));
 
-	if (gv_stream->priv->thread_data != NULL) {
+	if (priv->thread_data != NULL) {
 		ArvGvStreamThreadData *thread_data;
 		char *statistic_string;
 
-		thread_data = gv_stream->priv->thread_data;
+		thread_data = priv->thread_data;
 
 		statistic_string = arv_statistic_to_string (thread_data->statistic);
 		arv_debug_stream (statistic_string);
@@ -1344,8 +1363,6 @@ arv_gv_stream_finalize (GObject *object)
 		g_clear_object (&thread_data->device_socket_address);
 		g_clear_object (&thread_data->interface_socket_address);
 		g_clear_object (&thread_data->socket);
-		g_clear_object (&thread_data->gv_device);
-		g_clear_pointer (&thread_data->wakeup, arv_wakeup_free);
 
 		g_clear_pointer (&thread_data, g_free);
 	}
@@ -1359,6 +1376,7 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (gv_stream_class);
 	ArvStreamClass *stream_class = ARV_STREAM_CLASS (gv_stream_class);
 
+	object_class->constructed = arv_gv_stream_constructed;
 	object_class->finalize = arv_gv_stream_finalize;
 	object_class->set_property = arv_gv_stream_set_property;
 	object_class->get_property = arv_gv_stream_get_property;
@@ -1401,8 +1419,8 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 		object_class, ARV_GV_STREAM_PROPERTY_PACKET_TIMEOUT,
 		g_param_spec_uint ("packet-timeout", "Packet timeout",
 				   "Packet timeout, in µs",
-				   1000,
-				   10000000,
+				   0,
+				   G_MAXUINT,
 				   ARV_GV_STREAM_PACKET_TIMEOUT_US_DEFAULT,
 				   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
 		);
@@ -1410,8 +1428,8 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 		object_class, ARV_GV_STREAM_PROPERTY_FRAME_RETENTION,
 		g_param_spec_uint ("frame-retention", "Frame retention",
 				   "Packet retention, in µs",
-				   1000,
-				   10000000,
+				   0,
+				   G_MAXUINT,
 				   ARV_GV_STREAM_FRAME_RETENTION_US_DEFAULT,
 				   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
 		);
