@@ -271,7 +271,7 @@ main (int argc, char **argv)
 	else
 		g_print ("Looking for camera '%s'\n", arv_option_camera_name);
 
-	camera = arv_camera_new (arv_option_camera_name);
+	camera = arv_camera_new (arv_option_camera_name, &error);
 	if (camera != NULL) {
 		void (*old_sigint_handler)(int);
 		gint payload;
@@ -352,8 +352,8 @@ main (int argc, char **argv)
 			printf ("uv bandwidth limit     = %d [%d..%d]\n", arv_camera_uv_get_bandwidth (camera, NULL), min, max);
 		}
 
-		stream = arv_camera_create_stream (camera, stream_cb, NULL);
-		if (stream != NULL) {
+		stream = arv_camera_create_stream (camera, stream_cb, NULL, &error);
+		if (ARV_IS_STREAM (stream)) {
 			if (ARV_IS_GV_STREAM (stream)) {
 				if (arv_option_auto_socket_buffer)
 					g_object_set (stream,
@@ -418,21 +418,30 @@ main (int argc, char **argv)
 
 			arv_stream_get_statistics (stream, &n_completed_buffers, &n_failures, &n_underruns);
 
-			printf ("Completed buffers = %Lu\n", (unsigned long long) n_completed_buffers);
-			printf ("Failures          = %Lu\n", (unsigned long long) n_failures);
-			printf ("Underruns         = %Lu\n", (unsigned long long) n_underruns);
+			printf ("Completed buffers = %llu\n", (unsigned long long) n_completed_buffers);
+			printf ("Failures          = %llu\n", (unsigned long long) n_failures);
+			printf ("Underruns         = %llu\n", (unsigned long long) n_underruns);
 
 			arv_camera_stop_acquisition (camera, NULL);
 
 			arv_stream_set_emit_signals (stream, FALSE);
 
 			g_object_unref (stream);
-		} else
-			printf ("Can't create stream thread (check if the device is not already used)\n");
+		} else {
+			printf ("Can't create stream thread%s%s\n",
+				error != NULL ? ": " : "",
+				error != NULL ? error->message : "");
+
+			g_clear_error (&error);
+		}
 
 		g_object_unref (camera);
-	} else
-		printf ("No camera found\n");
+	} else {
+		printf ("No camera found%s%s\n",
+			error != NULL ? ": " : "",
+			error != NULL ? error->message : "");
+		g_clear_error (&error);
+	}
 
 	if (data.chunks != NULL)
 		g_strfreev (data.chunks);
