@@ -117,12 +117,14 @@ static void
 stream_test (void)
 {
 	ArvStream *stream;
+	GError *error = NULL;
 	size_t payload;
 	unsigned buffer_count = 0;
 	unsigned i;
 
-	stream = arv_camera_create_stream (camera, NULL, NULL);
+	stream = arv_camera_create_stream (camera, NULL, NULL, &error);
 	g_assert (ARV_IS_STREAM (stream));
+	g_assert (error == NULL);
 
 	payload = arv_camera_get_payload (camera, NULL);
 
@@ -162,12 +164,14 @@ static void
 dynamic_roi_test (void)
 {
 	ArvStream *stream;
+	GError *error = NULL;
 	size_t payload;
 	unsigned buffer_count = 0;
 	unsigned i, j;
 
-	stream = arv_camera_create_stream (camera, NULL, NULL);
+	stream = arv_camera_create_stream (camera, NULL, NULL, &error);
 	g_assert (ARV_IS_STREAM (stream));
+	g_assert (error == NULL);
 
 	payload = arv_camera_get_payload (camera, NULL);
 
@@ -226,7 +230,7 @@ main (int argc, char *argv[])
 
 	simulator = arv_gv_fake_camera_new ("lo", NULL);
 
-	camera = arv_camera_new ("Aravis-GV01");
+	camera = arv_camera_new ("Aravis-GV01", NULL);
 	g_assert (ARV_IS_CAMERA (camera));
 
 	g_test_add_func ("/fakegv/device_registers", register_test);
