@@ -37,10 +37,12 @@ node_value_type_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 	int i;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -72,11 +74,13 @@ integer_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 	gint64 v_int64;
 	const char *v_string;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -128,11 +132,13 @@ indexed_test (void)
 	ArvGc *genicam;
 	ArvGcNode *node;
 	ArvGcNode *selector;
+	GError *error = NULL;
 	gint64 v_int64;
 	double v_double;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", NULL);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -257,11 +263,13 @@ boolean_test (void)
 	ArvGc *genicam;
 	ArvGcNode *node;
 	ArvGcNode *node_b;
+	GError *error = NULL;
 	gboolean v_boolean;
 	const char *v_string;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -305,11 +313,13 @@ float_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 	double v_double;
 	const char *v_string;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -360,13 +370,15 @@ enumeration_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 	gint64 v_int64;
 	gint64 *values;
 	guint n_values;
 	const char *v_string;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -379,7 +391,7 @@ enumeration_test (void)
 	v_int64 = arv_gc_enumeration_get_int_value (ARV_GC_ENUMERATION (node), NULL);
 	g_assert_cmpint (v_int64, ==, 0);
 
-	values = arv_gc_enumeration_get_available_int_values (ARV_GC_ENUMERATION (node), &n_values, NULL);
+	values = arv_gc_enumeration_dup_available_int_values (ARV_GC_ENUMERATION (node), &n_values, NULL);
 	g_assert_cmpint (n_values, ==, 2);
 	g_assert (values != NULL);
 	g_free (values);
@@ -415,10 +427,12 @@ swiss_knife_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 	gint64 value;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -462,11 +476,13 @@ converter_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 	double v_double;
 	gint64 v_int64;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -530,8 +546,9 @@ register_test (void)
 	gint64 value;
 	double value_f;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -758,9 +775,11 @@ mandatory_test (void)
 	ArvDevice *device;
 	ArvGc *genicam;
 	ArvGcNode *node;
+	GError *error = NULL;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	genicam = arv_device_get_genicam (device);
 	g_assert (ARV_IS_GC (genicam));
@@ -867,8 +886,9 @@ chunk_data_test (void)
 	size_t size;
 	size_t chunk_data_size;
 
-	device = arv_fake_device_new ("TEST0");
+	device = arv_fake_device_new ("TEST0", &error);
 	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
 
 	parser = arv_device_create_chunk_parser (device);
 	g_assert (ARV_IS_CHUNK_PARSER (parser));
@@ -933,6 +953,40 @@ chunk_data_test (void)
 	g_object_unref (device);
 }
 
+static void
+visibility_test (void)
+{
+	ArvDevice *device;
+	ArvGc *genicam;
+	ArvGcNode *node;
+	GError *error = NULL;
+
+	device = arv_fake_device_new ("TEST0", &error);
+	g_assert (ARV_IS_FAKE_DEVICE (device));
+	g_assert (error == NULL);
+
+	genicam = arv_device_get_genicam (device);
+	g_assert (ARV_IS_GC (genicam));
+
+	node = arv_gc_get_node (genicam, "RWInteger");
+	g_assert (ARV_IS_GC_FEATURE_NODE (node));
+	g_assert_cmpint (arv_gc_feature_node_get_visibility (ARV_GC_FEATURE_NODE (node)), ==, ARV_GC_VISIBILITY_INVISIBLE);
+
+	node = arv_gc_get_node (genicam, "RWFloat");
+	g_assert (ARV_IS_GC_FEATURE_NODE (node));
+	g_assert_cmpint (arv_gc_feature_node_get_visibility (ARV_GC_FEATURE_NODE (node)), ==, ARV_GC_VISIBILITY_GURU);
+
+	node = arv_gc_get_node (genicam, "RWBoolean");
+	g_assert (ARV_IS_GC_FEATURE_NODE (node));
+	g_assert_cmpint (arv_gc_feature_node_get_visibility (ARV_GC_FEATURE_NODE (node)), ==, ARV_GC_VISIBILITY_EXPERT);
+
+	node = arv_gc_get_node (genicam, "Enumeration");
+	g_assert (ARV_IS_GC_FEATURE_NODE (node));
+	g_assert_cmpint (arv_gc_feature_node_get_visibility (ARV_GC_FEATURE_NODE (node)), ==, ARV_GC_VISIBILITY_BEGINNER);
+
+	g_object_unref (device);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -954,6 +1008,7 @@ main (int argc, char *argv[])
 	g_test_add_func ("/genicam/mandatory", mandatory_test);
 	g_test_add_func ("/genicam/chunk-data", chunk_data_test);
 	g_test_add_func ("/genicam/indexed", indexed_test);
+	g_test_add_func ("/genicam/visibility", visibility_test);
 
 	result = g_test_run();
 
