@@ -889,8 +889,8 @@ start_video (ArvViewer *viewer)
 	stop_video (viewer);
 
 	viewer->rotation = 0;
-	viewer->stream = arv_camera_create_stream (viewer->camera, stream_cb, NULL);
-	if (viewer->stream == NULL) {
+	viewer->stream = arv_camera_create_stream (viewer->camera, stream_cb, NULL, NULL);
+	if (!ARV_IS_STREAM (viewer->stream)) {
 		g_object_unref (viewer->camera);
 		viewer->camera = NULL;
 		return FALSE;
@@ -922,8 +922,6 @@ start_video (ArvViewer *viewer)
 	arv_camera_get_exposure_time_bounds (viewer->camera, &viewer->exposure_min, &viewer->exposure_max, NULL);
 	arv_camera_get_gain_bounds (viewer->camera, &gain_min, &gain_max, NULL);
 	frame_rate = arv_camera_get_frame_rate (viewer->camera, NULL);
-	auto_gain = arv_camera_get_gain_auto (viewer->camera, NULL) != ARV_AUTO_OFF;
-	auto_exposure = arv_camera_get_gain_auto (viewer->camera, NULL) != ARV_AUTO_OFF;
 
 	is_frame_rate_available = arv_camera_is_frame_rate_available (viewer->camera, NULL);
 	is_exposure_available = arv_camera_is_exposure_time_available (viewer->camera, NULL);
@@ -1121,7 +1119,7 @@ start_camera (ArvViewer *viewer, const char *camera_id)
 
 	stop_camera (viewer);
 
-	viewer->camera = arv_camera_new (camera_id);
+	viewer->camera = arv_camera_new (camera_id, NULL);
 
 	if (!ARV_IS_CAMERA (viewer->camera))
 		return FALSE;
@@ -1141,8 +1139,8 @@ start_camera (ArvViewer *viewer, const char *camera_id)
 	list_store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (viewer->pixel_format_combo)));
 	gtk_list_store_clear (list_store);
 	n_valid_formats = 0;
-	pixel_format_strings = arv_camera_get_available_pixel_formats_as_strings (viewer->camera, &n_pixel_format_strings, NULL);
-	pixel_formats = arv_camera_get_available_pixel_formats (viewer->camera, &n_pixel_formats, NULL);
+	pixel_format_strings = arv_camera_dup_available_pixel_formats_as_strings (viewer->camera, &n_pixel_format_strings, NULL);
+	pixel_formats = arv_camera_dup_available_pixel_formats (viewer->camera, &n_pixel_formats, NULL);
 	g_assert (n_pixel_formats == n_pixel_format_strings);
 	pixel_format_string = arv_camera_get_pixel_format_as_string (viewer->camera, NULL);
 	for (i = 0; i < n_pixel_formats; i++) {
